@@ -1,18 +1,32 @@
 """Implementation of ProteinSequenceDataset."""
 import logging
+from typing import Dict, Iterable
 
+import numpy as np
 import torch
+from importlib_resources import files
+
+from pytoda.warnings import device_warning
 
 from ..proteins.protein_feature_language import ProteinFeatureLanguage
 from ..proteins.protein_language import ProteinLanguage
-from ..proteins.transforms import SequenceToTokenIndexes
+from ..proteins.transforms import (
+    MutateResidues,
+    ProteinAugmentFlipSubstrs,
+    ProteinAugmentSwapSubstrs,
+    ReplaceByFullProteinSequence,
+    SequenceToTokenIndexes,
+)
 from ..transforms import (
     AugmentByReversing,
     Compose,
+    DiscardLowercase,
+    ExtractFromDict,
     LeftPadding,
     ListToTensor,
     Randomize,
     ToTensor,
+    ToUpperCase,
 )
 from ._fasta_eager_dataset import _FastaEagerDataset
 from ._fasta_lazy_dataset import _FastaLazyDataset
@@ -87,7 +101,15 @@ SEQUENCE_DATASET_IMPLEMENTATIONS = {  # get class and acceptable keywords
 def protein_sequence_dataset(
     *filepaths: str, filetype: str, backend: str, **kwargs
 ) -> KeyDataset:
-    """Return a dataset of protein sequences."""
+    """
+    Return a dataset of protein sequences.
+
+    Args:
+        filepaths (Iterable[str]): Paths to the files containing the protein sequences.
+        filetype (str): The filetype of the protein sequences.
+        backend (str): The backend to use for the dataset.
+        kwargs: Keyword arguments for the dataset.
+    """
     try:
         # hardcoded factory
         dataset_class, valid_keys = SEQUENCE_DATASET_IMPLEMENTATIONS[filetype][backend]
@@ -107,10 +129,7 @@ def protein_sequence_dataset(
 
 
 class ProteinSequenceDataset(DatasetDelegator):
-    """
-    Protein Sequence dataset using a Language to transform sequences.
-
-    """
+    """Protein Sequence dataset using a Language to transform sequences."""
 
     def __init__(
         self,
@@ -122,13 +141,13 @@ class ProteinSequenceDataset(DatasetDelegator):
         padding_length: int = None,
         add_start_and_stop: bool = False,
         augment_by_revert: bool = False,
+        sequence_augment: Dict = {},
+        protein_keep_only_uppercase: bool = False,
         randomize: bool = False,
-        device: torch.device = (
-            torch.device('cuda' if torch.cuda.is_available() else 'cpu')
-        ),
         backend: str = 'eager',
         iterate_dataset: bool = False,
         name: str = 'protein-sequences',
+        device: torch.device = None,
         **kwargs,
     ) -> None:
         """
@@ -153,17 +172,52 @@ class ProteinSequenceDataset(DatasetDelegator):
                 Sequences. Defaults to False.
             randomize (bool): perform a true randomization of Protein tokens.
                 Defaults to False.
-            device (torch.device): device where the tensors are stored.
-                Defaults to gpu, if available.
+            sequence_augment (Dict): a dictionary to specify additional sequence
+                augmentation. Defaults to {}. Items can be:
+                    `alignment_path`: A path (str) to a `.smi` (or `.tsv`) file
+                        with alignment information that specifies which residues of the
+                        sequence are to be used. E.g., to extract active site sequences
+                        from full proteins. Do not use a header in the file.
+                        1. column has to be the full protein sequence (use upper
+                            case only for residues to be used). E.g., ggABCggDEFgg
+                        2. column has to be the condensed sequence (ABCDEF).
+                        3. column has to be the protein identifier.
+                        NOTE: Such a file is *necessary* to apply all augmentation types
+                        specified in this dictionary (`sequence_augment`).
+                        NOTE: Unless specified, this defaults to
+                        `kinase_activesite_alignment.smi`, a file in
+                        `pytoda.proteins.metadata` that can ONLY be used to
+                        extract active site sequences of human kinases (based on the
+                        active site definition in Sheridan et al. (2009, JCIM) and
+                        Martin & Mukherjee (2012, JCIM).
+                    `discard_lowercase`: A (bool) specifying whether all lowercase
+                        characters (residues) in the sequence should be discarded.
+                        NOTE: This defaults to True.
+                    `flip_substrings`: A probability (float) to flip each contiguous
+                        upper-case substring (e.g., an active site substring that lies
+                        contiguously in the original sequence).
+                        Defaults to 0.0, i.e., no flipping.
+                        E.g., ABCDEF could become CBADEF or CBAFED or ABCFED if the
+                        original sequence is ggABCggDEFgg.
+                    `swap_substrings`: A probability (float) to swap neighboring
+                        substrings. Defaults to 0.0, i.e., no swapping.
+                        E.g., ABCDEF could become DEFABC if the original sequence is
+                        ggABCggDEFgg.
+                    `noise`: A 2-Tuple of (float, float) that specifies the probability
+                        for a random, single-residue mutation inside and outside the
+                        relevent part. Defaults to (0.0, 0.0), i.e., no noise.
+                        E.g., with (0.0, 0.5),  ggABCggDEFgg could become hgABCgbDEFgg.
             iterate_dataset (bool): whether to go through all items in the dataset
                 to detect unknown characters, find longest sequence and checks
                 passed padding length if applicable. Defaults to False.
             backend (str): memory management backend.
                 Defaults to eager, prefer speed over memory consumption.
             name (str): name of the ProteinSequenceDataset.
+            device (torch.device): DEPRECATED
             kwargs (dict): additional arguments for dataset constructor.
         """
 
+        device_warning(device)
         # Parse language object and data paths
         self.filepaths = filepaths
         self.filetype = filetype
@@ -179,6 +233,8 @@ class ProteinSequenceDataset(DatasetDelegator):
 
         # setup dataset
         self._setup_dataset(**kwargs)
+        self.cumulative_sizes = np.cumsum([0] + [len(s) for s in self.datasets])
+
         DatasetDelegator.__init__(self)  # delegate to self.dataset
         if self.has_duplicate_keys:
             raise KeyError(f'Please remove duplicates from your {self.filetype} file.')
@@ -229,12 +285,14 @@ class ProteinSequenceDataset(DatasetDelegator):
             )
         self.randomize = randomize
         self.augment_by_revert = augment_by_revert
-        self.device = device
 
-        # Build up cascade of Protein transformations
-        transforms = []
+        # Sequence-based augmentation
+        self.sequence_augment = sequence_augment
+        transforms = self.setup_sequence_augmentation(sequence_augment)
+
         if self.augment_by_revert:
             transforms += [AugmentByReversing()]
+
         self.language_transforms = Compose(transforms.copy())
 
         transforms += [SequenceToTokenIndexes(protein_language=self.protein_language)]
@@ -248,9 +306,9 @@ class ProteinSequenceDataset(DatasetDelegator):
                 )
             ]
         if isinstance(self.protein_language, ProteinFeatureLanguage):
-            transforms += [ListToTensor(device=self.device)]
+            transforms += [ListToTensor()]
         elif isinstance(self.protein_language, ProteinLanguage):
-            transforms += [ToTensor(device=self.device)]
+            transforms += [ToTensor()]
         else:
             raise TypeError(
                 'Please choose either ProteinLanguage or ' 'ProteinFeatureLanguage'
@@ -263,6 +321,68 @@ class ProteinSequenceDataset(DatasetDelegator):
             *self.filepaths, filetype=self.filetype, backend=self.backend, **kwargs
         )
 
+    def setup_sequence_augmentation(self, sequence_augment: Dict):
+        """
+        Setup the sequence augmentation.
+
+        Args:
+            sequence_augment: A dictionary to specify the sequence augmentation
+                strategies. For details see the constructor docs.
+        """
+        if not isinstance(sequence_augment, Dict):
+            raise TypeError(
+                f"Pass sequence_augment as Dict not {type(sequence_augment)}"
+            )
+
+        # Build up cascade of Protein transformations
+        transforms = []
+
+        # No work to do
+        if sequence_augment == {}:
+            transforms += [ExtractFromDict(key='sequence')]
+            return transforms
+
+        # Start composing transforms
+        self.alignment_path = sequence_augment.pop(
+            'alignment_path',
+            files('pytoda.proteins.metadata').joinpath(
+                'kinase_activesite_alignment.smi'
+            ),
+        )
+
+        transforms += [ReplaceByFullProteinSequence(self.alignment_path)]
+
+        # Flip substrings
+        self.flip_substrings = sequence_augment.pop('flip_substrings', 0.0)
+        transforms += [ProteinAugmentFlipSubstrs(p=self.flip_substrings)]
+
+        # Swap substrings
+        self.swap_substrings = sequence_augment.pop('swap_substrings', 0.0)
+        transforms += [ProteinAugmentSwapSubstrs(self.swap_substrings)]
+
+        # Inject noise (single residue mutation)
+        self.noise = sequence_augment.pop('noise', (0.0, 0.0))
+        if not isinstance(self.noise, Iterable) or len(self.noise) != 2:
+            raise TypeError(f'Noise has to be Iterable of length 2 not {self.noise}')
+        transforms += [MutateResidues(*self.noise)]
+
+        # Prune lowercase characters
+        self.discard_lowercase = sequence_augment.pop('discard_lowercase', True)
+        if self.discard_lowercase:
+            transforms += [DiscardLowercase()]
+
+        """
+        In the future this could be removed to distinguish between upper/lowercase
+        characters but it would necessitate changes in protein language.
+        """
+        # Always convert to uppercase
+        transforms += [ToUpperCase()]
+
+        for k, v in sequence_augment.items():
+            logger.warning(f"Ignoring sequence_augment item: {k}: {v}.")
+
+        return transforms
+
     def __getitem__(self, index: int) -> torch.Tensor:
         """
         Generates one sample of data.
@@ -274,4 +394,5 @@ class ProteinSequenceDataset(DatasetDelegator):
             torch.Tensor: a torch tensor of token indexes,
                 for the current sample.
         """
-        return self.transform(self.dataset[index])
+        sample_dict = dict(sequence=self.dataset[index], id=self.dataset.get_key(index))
+        return self.transform(sample_dict)
