@@ -28,9 +28,12 @@ import torch
 from selfies import decoder as selfies_decoder
 from selfies import encoder as selfies_encoder
 
+from pytoda.warnings import device_warning
+
 from ..files import read_smi
 from ..transforms import Compose
 from ..types import (
+    Any,
     Files,
     Indexes,
     Iterable,
@@ -176,8 +179,17 @@ class SMILESLanguage(object):
         warnings.warn(
             "Loading languages will use a text files in the future", FutureWarning
         )
-        with open(filepath, 'rb') as f:
-            smiles_language = dill.load(f)
+        try:
+            with open(filepath, 'rb') as f:
+                smiles_language = dill.load(f)
+        except TypeError:
+            # Necessary to load python3.7 pickled objects with >=3.8:
+            # For details see: https://github.com/uqfoundation/dill/pull/406
+            storage = dill._dill._reverse_typemap['CodeType']
+            dill._dill._reverse_typemap['CodeType'] = dill._dill._create_code
+            with open(filepath, 'rb') as f:
+                smiles_language = dill.load(f)
+            dill._dill._reverse_typemap['CodeType'] = storage
         return smiles_language
 
     @staticmethod
@@ -743,9 +755,7 @@ class SMILESTokenizer(SMILESLanguage):
         add_start_and_stop: bool = False,
         padding: bool = False,
         padding_length: int = None,
-        device: torch.device = torch.device(
-            'cuda' if torch.cuda.is_available() else 'cpu'
-        ),
+        device: Any = None,
     ) -> None:
         """
         Initialize SMILES language.
@@ -788,8 +798,7 @@ class SMILESTokenizer(SMILESLanguage):
                 applies only if padding is True. See `set_max_padding` to set
                 it to longest token sequence the smiles language encountered.
                 Defaults to None.
-            device (torch.device): device where the tensors are stored.
-                Defaults to gpu, if available.
+            device (Any): Deprecated argument that will be removed in the future.
 
         NOTE:
             See `set_smiles_transforms` and `set_encoding_transforms` to change
@@ -797,6 +806,7 @@ class SMILESTokenizer(SMILESLanguage):
             `reset_initial_transforms`. Assignment of class attributes
             in the parameter list will trigger such a reset.
         """
+        device_warning(device)
         super().__init__(
             name=name,
             smiles_tokenizer=smiles_tokenizer,
@@ -819,7 +829,9 @@ class SMILESTokenizer(SMILESLanguage):
         self.add_start_and_stop = add_start_and_stop
         self.padding = padding
         self.padding_length = padding_length
-        self.device = device
+
+        if device:
+            logger.warning(device_warning(device))
 
         self._init_attributes = [  # additions to init_kwargs for pretrained
             'canonical',
@@ -844,7 +856,6 @@ class SMILESTokenizer(SMILESLanguage):
 
         self._attributes_to_trigger_reset = [
             *self._init_attributes,
-            'device',
             'start_index',
             'stop_index',
         ]  # could be updated in inheritance
@@ -929,7 +940,6 @@ class SMILESTokenizer(SMILESLanguage):
             self.padding,
             self.padding_length,
             self.padding_index,
-            self.device,
         )
         self._set_token_len_fn(self.add_start_and_stop)
 
@@ -972,7 +982,6 @@ class SMILESTokenizer(SMILESLanguage):
         add_start_and_stop=None,
         padding=None,
         padding_length=None,
-        device=None,
     ):
         """Helper function to reversibly change steps of the transforms."""
         self.transform_encoding = compose_encoding_transforms(
@@ -987,7 +996,6 @@ class SMILESTokenizer(SMILESLanguage):
             if padding_length is not None
             else self.padding_length,
             padding_index=self.padding_index,
-            device=device if device is not None else self.device,
         )
         if add_start_and_stop is not None:
             self._set_token_len_fn(add_start_and_stop)
